require 'soar_idm/soar_idm'

module Soar
  module Registry
    module Identity
      module Provider
        module Staff

          class Uuid < SoarIdm::IdmApi

            ##
            # @param [Soar::Registry::Directory] identity_directory
            # @param [Soar::Registry::Directory] role_directory
            ##
            def initialize(identity_directory:, role_directory:)
              @identity_directory = identity_directory
              @role_directory = role_directory
            end

            ##
            # @param [String] identity_uuid
            # @return [Array<String>] list of roles
            ##
            def calculate_roles(identity_uuid)
              entries = @role_directory.directory.search(@role_directory.search_index, identity_uuid)
              roles = []
              entries.each do |entry|
                identity_source = entry['identity_source'] if entry.key?('identity_source') 
                roles << entry[@role_directory.fetch_index[1]]
              end
              return roles
            end

            ##
            # @param [Hash{String => String, Number}] identity
            # @return [Array<String,Number>] list of identifiers
            ##
            def calculate_identifiers(identity)
              indexes = @identity_directory.directory.index
              identifiers = []
              indexes.each { |index| 
                identifiers << identity[index.to_s]
              }
              identifiers
            end

            ## 
            # @param [String] identity_uuid
            # @param [String] role
            # @return [Hash{String => String, Number, Hash, Array}] A hash of attributes 
            ##
            def calculate_attributes(identity_uuid, role)
              primary_key = {
                @role_directory.fetch_index[0] => identity_uuid,
                @role_directory.fetch_index[1] => role
              }
              result = @role_directory.directory.fetch(primary_key)
              attributes = {
                role => result.key?('identity_role_attributes') ? result['identity_role_attributes'] : {}
              }
              return attributes
            end 

            ## 
            # @param [String] identity_uuid
            # @return [Hash{String => String, Number, Hash, Array}] A hash of attributes 
            ##
            def calculate_all_attributes(identity_uuid)
              begin
                role_entries = @role_directory.directory.search(@role_directory.search_index, identity_uuid)
                roles = {}
                role_entries.each { |role_entry|
                  roles[role_entry['identity_role']] = role_entry.key?('identity_role_attributes') ? role_entry['identity_role_attributes'] : {}
                }
                identity = @identity_directory.directory.fetch(identity_uuid)
                attributes = {}
                attributes["identity_uuid"] = identity_uuid 
                attributes["firstname"] = identity['givenName'] if identity.key?('givenName')
                attributes["lastname"] = identity['sn'] if identity.key?('sn')
                attributes["email"] = identity['mail'] if identity.key?('mail')
                attributes["roles"] = roles
                return attributes
              rescue Soar::Registry::Directory::Error::NoEntriesFoundError => e
                raise SoarIdm::IdentityError, e.message
              end
            end

            ##
            # @param [String] identity_uuid
            # @return [Array<String>] identifiers
            # @raise [Soar::Registry::Directory::Error::NoEntriesFoundError]
            ##
            def calculate_identities(identity_uuid)
              return [identity_uuid]
            end

          end
        end
      end
    end
  end
end
