require 'faker'
require 'soar/registry/identity'
require 'soar/registry/directory'
require 'soar/authentication/identity_uuid_translator/provider/customer'
require 'soar/authentication/identity_uuid_translator/uuid_generator'

module Soar
  module Registry
    module Identity
      module Test
        module OrchestrationProvider
          module Stub
            class Uuid

              def initialize
                Faker::UniqueGenerator.clear
                @identity_uuid = SecureRandom.uuid
              end

              def given_identity_directory
                @directory = Soar::Registry::Directory.new(
                  Soar::Registry::Directory::Provider::Stub.new(
                    table: 'identities',
                    index: ['identity_uuid']
                  )
                )
              end

              def given_roles_directory
                true
              end

              def given_roles
                @identity['roles'] = {
                  Faker::Company.unique.profession => {}
                }
                @roles = @identity['roles'].keys
              end

              def given_role_with_attributes
                @identity['roles'] = {
                  Faker::Company.unique.profession => {
                    Faker::Hacker.noun => Faker::Hacker.verb
                  }
                }
                @roles = @identity['roles'].keys
              end

              def given_identity
                @identity = {
                  "identity_uuid" => @identity_uuid,
                  "firstname" =>  Faker::Name.unique.first_name,
                  "lastname" => Faker::Name.unique.last_name,
                  "email" => Faker::Internet.unique.email
                }
              end

              def given_identity_registry
                @directory.put(@identity) if not @identity.nil?
                @idr = Soar::Registry::Identity.new(
                  Soar::Registry::Identity::Provider::Stub::Uuid.new(
                    directory: @directory
                  )
                )
              end

              ##### when

              def get_roles
                @result = @idr.get_roles(@identity_uuid) 
              end

              def get_role_attributes
                @result = @idr.get_attributes(@identity_uuid, @roles.nil? ? Faker::Company.unique.profession : @roles[0])
              end

              def get_identity_attributes
                begin
                  @result = @idr.get_attributes(@identity_uuid)
                rescue SoarIdm::IdentityError => e
                  @error = e
                end
              end

              ##### then

              def identity_attributes?
                @result == @identity
              end

              def nil?
                @result == nil
              end
              
              def role_with_empty_attributes?
                @result == {
                  @roles[0] => {}
                }
              end

              def role_with_attributes?
                role_with_attributes = {
                  @roles[0] => @identity['roles'][@roles[0]]
                }
                @result == role_with_attributes
              end

              def roles?
                @result.sort == @roles.sort
              end

              def no_roles?
                @result == []
              end

              def identity_error?
                @error.is_a?(SoarIdm::IdentityError)
              end

            end
          end
        end
      end
    end
  end
end
